# -*- coding: utf-8 -*-
"""Main Controller"""

from tg import expose, flash, require, url, lurl
from tg import request, redirect, tmpl_context
from tg.i18n import ugettext as _, lazy_ugettext as l_
from tg.exceptions import HTTPFound

from hoyataxa.lib.base import BaseController
from hoyataxa.controllers.error import ErrorController

__all__ = ['RootController']

from hoyataxa.model.taxa import *

initdb()

import tw2.forms as twf

from tg import validate
from formencode import validators, compound, schema

class SearchFormValidator(schema.Schema):
    query = compound.All(validators.Regex(r'^ *[A-Za-z]* *$'),
                         validators.PlainText(),
                         validators.String(min=3,strip=True))
    mode  = validators.OneOf(["Starts with","Ends with","Contains"])
    limit = validators.Int(min=1)  


class SearchForm(twf.Form):
    class child(twf.TableLayout):
        query = twf.TextField(label="Search Term")
        mode = twf.SingleSelectField(label="Seach Mode",
                                     options=["Contains",
                                              "Starts with",
                                              "Ends with"],
                                     prompt_text=None)
        limit = twf.TextField(label="Max. Names")
        # for presentation only...
        css_class = 'table'
        attrs = {'style': 'width: 600px;'}
    
    action = '/search'
    submit = twf.SubmitButton(value="Search")
    validator = SearchFormValidator

class RootController(BaseController):

    @expose('hoyataxa.templates.index')
    def index(self,**kwargs):
        """Handle the front-page."""
        return dict(title="Hoya Taxa!",
                    form=SearchForm)

    @expose('hoyataxa.templates.search')
    @expose('hoyataxa.templates.searchxml',
            content_type='text/xml')
    @validate(SearchForm, error_handler=index)
    def search(self,query,mode=None,limit=None):
        if limit == None:
            limit = 1000
        if mode == None:
            mode = "Contains"
        if mode == 'Starts with':
            names = Name.select(Name.q.name.startswith(query))
        elif mode == 'Ends with':
            names = Name.select(Name.q.name.endswith(query))
        elif mode == 'Contains':
            names = Name.select(Name.q.name.contains(query))
        return dict(names=names,query=query,mode=mode,limit=limit)

    @expose('hoyataxa.templates.taxa')
    @expose('hoyataxa.templates.taxaxml',
            content_type='text/xml')
    def taxa(self,taxid):
        t = Taxonomy.byTaxid(int(taxid))
        lineage = []
        t1 = t
        while t1.taxid != 1:
            if t1.rank != 'no rank':
                lineage.append(t1)
            t1 = t1.parent
        lineage.reverse()
        return dict(title=t.scientific_name,
                    taxa=t, lineage=lineage)



